#!/usr/bin/env bats

# Test info-111


## Setup: source travo as a library and authenticate on the test server
#export TRAVO_GITLAB="https://gitlab.u-psud.fr/"
SCRIPT=/home/nthiery/Enseignement/Info111/binder/info-111
. ${SCRIPT}

#travo_gitlab_authenticate
user=$(travo_gitlab_username)
echo $user > /tmp/truc

## Test project creation

assignment=Semaine1
repo="Info111/2020-2021/${assignment}"
personal_repo="${user}/${assignment}"
tmpdir="/tmp/tmpdir" # Must be absolute
rm -rf ${tmpdir}
mkdir ${tmpdir}
cd ${tmpdir}
export COURSE_STUDENT_DIR=${tmpdir}

@test "check_dir" {
    run check_dir
    [ "$status" -eq 0 ]

    COURSE_STUDENT_DIR=/tmp run check_dir
    [ "$status" -eq 1 ]
}

@test "assignment_repo_path" {
    run assignment_repo_path "${assignment}"
    [ "$status" -eq 0 ]
    [ "$output" == "${repo}" ]
}

@test "assignment_repo_http_url" {
    run assignment_repo_http_url "${assignment}"
    [ "$status" -eq 0 ]
    echo "|${output}|"
    [ "$output" == "https://gitlab.u-psud.fr/Info111/2020-2021/Semaine1.git" ]
}

@test "assignment_personal_repo_oauth2_http_url" {
    run assignment_personal_repo_oauth2_http_url "${assignment}"
    [ "$status" -eq 0 ]
    echo $output
    [ "$output" == "https://oauth2@gitlab.u-psud.fr/nicolas.thiery/Semaine1.git" ]
}

@test "Fetch fresh assignment, create personal, refetch" {
    # hardcoded path for safety for now
    assignment_personal_repo ${assignment} &&
        travo_gitlab_remove_project nicolas.thiery/Semaine1

    run ${SCRIPT} fetch ${assignment}
    [ "$status" -eq 0 ]
    [[ "${output}" =~ "git clone" ]]
    [ -d ${assignment} ]
    output=$(cd ${assignment} && git remote -v)
    pattern="origin	${TRAVO_GITLAB}${repo}"
    [[ "${output}" =~ "${pattern}" ]]

    assignment_ensure_personal_repo_created "${assignment}"

    run ${SCRIPT} fetch ${assignment}
    [ "$status" -eq 0 ]
    [[ "${output}" =~ "Reconfiguration" ]]
    [[ "${output}" =~ "git remote" ]]
    [[ "${output}" =~ "git pull" ]]
    [[ ! ( "${output}" =~ "git clone" ) ]]

    cd ${assignment} && run git remote -v
    [ "$status" -eq 0 ]
    pattern="origin	${TRAVO_GITLAB}${personal_repo}"
    [[ "${output}" =~ "${pattern}" ]]
}

@test "Fetch existing assignment" {
    assignment_ensure_personal_repo_created "${assignment}"

    run ${SCRIPT} fetch ${assignment}
    [ "$status" -eq 0 ]
    [[ "${output}" =~ "git clone" ]]
    [ -d ${assignment} ]
    cd ${assignment} && run git remote -v
    [ "$status" -eq 0 ]
    pattern="origin	${TRAVO_GITLAB}${personal_repo}"
    [[ "${output}" =~ "${pattern}" ]]
}

@test "Conflict resolution" {
    assignment_ensure_personal_repo_created "${assignment}"

    ${SCRIPT} fetch ${assignment}
    (
        cd ${assignment} &&
        echo change1 >> change1 &&
        git add change1 &&
        git commit -m "change1"
    )

    # Fetch elsewhere and create a distinct change
    othertmpdir="${tmpdir}/other"
    mkdir ${othertmpdir}
    (
        cd ${othertmpdir} &&
        COURSE_STUDENT_DIR=${othertmpdir} ${SCRIPT} fetch ${assignment} &&
        cd ${assignment} &&
        echo change2 >> change2 &&
        git add change2 &&
        git commit -m "change2" &&
        assignment_push ${assignment}
    )

    run ${SCRIPT} fetch ${assignment}
    [ "$status" -eq 0 ]
    [[ "${output}" =~ "renommé" ]]

    run ls ${assignment}-202-*
    [ "$status" -eq 0 ]
}
